__author__ = "hugo.inzirillo"

from dataclasses import dataclass


@dataclass
class EodQuotePk:
    productCode: str = None
    date: str = None

    def __hash__(self):
        return hash((self.productCode, self.date))

    def __eq__(self, other):
        return self.productCode == other.productCode and self.date == other.date \
            if isinstance(other, self.__class__) else False

    def __le__(self, other):
        return self.date <= other.date

    def __ge__(self, other):
        return self.date >= other.date

    def __lt__(self, other):
        return self.date < other.date

    def __gt__(self, other):
        return self.date > other.date


@dataclass
class EodQuote:
    pk: EodQuotePk = None
    open: float = None
    close: float = None
    low: float = None
    high: float = None
    totalReturnClose: float = None
    volume: float = None
    marketCap: float = None
    adjMarketCap: float = None
    adjClose: float = None
    divisor: float = None

    def __hash__(self):
        return hash(self.pk)

    def __eq__(self, other):
        return self.pk == other.pk if isinstance(other, self.__class__) else False

    def from_dict(self, dict_object: dict):
        if isinstance(dict_object, dict):
            self.pk = EodQuotePk(productCode=dict_object["productCode"],
                                 date=dict_object["date"])

            self.close = dict_object["close"]
            self.totalReturnClose = dict_object["totalReturnClose"]
            return self

    def __le__(self, other):
        return self.pk <= other.pk

    def __ge__(self, other):
        return self.pk >= other.pk

    def __lt__(self, other):
        return self.pk < other.pk

    def __gt__(self, other):
        return self.pk > other.pk


__author__ = "hugo.inzirillo"

from napoleontoolbox.dataloader.models import Ressources, Provider, Service

__all__ = ['Ressources',
           'Service',
           'Provider']


class DataLoader(object):
    def __init__(self):
        self.__ressources = Ressources()

    @property
    def ressources(self):
        return self.__ressources



__author__ = "hugo.inzirillo"

from pandas import Timestamp

from models.filter import SignalComputationFilter
from utils.service_exceptions import NapbotsReportingServiceValueError, \
    NapbotsReportingServiceException


class SignalComputationFilterValidator(object):
    def __init__(self, signal_filter: SignalComputationFilter):
        self.__signal_filter = signal_filter

    @property
    def signal_filter(self) -> SignalComputationFilter:
        return self.__signal_filter

    @property
    def is_valid(self) -> bool:
        return self.__validate()

    def __validate(self):
        self.__check_timestamp()

        return True

    def __check_timestamp(self):
        if self.signal_filter.lastOnly:
            pass
        elif self.signal_filter.lastOnly == False \
                and self.signal_filter.maxTs is not None \
                and self.signal_filter.minTs is not None:
            if isinstance(self.signal_filter.maxTs, Timestamp) and isinstance(self.signal_filter.minTs, Timestamp):
                self.__check_date(self.signal_filter.minTs,self.signal_filter.maxTs)
                self.signal_filter.maxTs = self.signal_filter.maxTs.strftime("%Y-%m-%dT%H:%M:%S")
                self.signal_filter.minTs = self.signal_filter.minTs.strftime("%Y-%m-%dT%H:%M:%S")
            else:
                self.__to_ts()

    def __to_ts(self):
        try:
            min_ts = Timestamp(self.__signal_filter.minTs)
            max_ts = Timestamp(self.__signal_filter.maxTs)
            self.__check_date(min_ts, max_ts)
        except NapbotsReportingServiceValueError as ve:
            raise ve

    def __check_date(self, min_ts: Timestamp, max_ts: Timestamp):

        time_dela = max_ts - min_ts
        if max_ts < min_ts:
            raise NapbotsReportingServiceException("Max TimeStamp should be after Min TimeStamp")
        if len(self.__signal_filter.productCodes) == 1 and time_dela.days < 30:
            pass



import numpy as np
import pandas as pd
from collections import Counter
from heapq import nlargest
from numba import njit, jit
from numba.typed import Dict,List


# def most_correlated_features(X,threshold):
#     generic_features = ['close','high', 'low', 'open', 'volume']
#     columns_to_compute = [col for col in X.columns if col not in generic_features]
#     output_correlations = []
#     for i in range(len(columns_to_compute)):
#         for j in range(i+1,len(columns_to_compute)):
#             left_feature = columns_to_compute[i]
#             right_feature=columns_to_compute[j]
#             if left_feature != right_feature:
#                 # print(left_feature, right_feature)
#                 # print(X[left_feature].nunique())
#                 # print('done')
#                 correlation_matrix = np.corrcoef(X[left_feature],X[right_feature])
#                 output_correlations.append(
#                     {
#                         'daily_feature_1':left_feature,
#                         'daily_feature_2':right_feature,
#                         'correlation':correlation_matrix[0][1]
#                     }
#                 )
#
#     output_correlations_df = pd.DataFrame(output_correlations)
#     output_correlations_df['abs_correlation'] = abs(output_correlations_df['correlation'] )
#     output_correlations_df = output_correlations_df.sort_values(by='abs_correlation', ascending=False)
#     columns_to_drop = output_correlations_df.loc[output_correlations_df['abs_correlation'] > threshold]['daily_feature_2'].unique()
#     return columns_to_drop

def rescalize(y, range1, range2):
    delta1 = range1[1] - range1[0]
    delta2 = range2[1] - range2[0]
    return (delta2 * (y - range1[0]) / delta1) + range2[0]

@njit()
def compute_features_correlation(X_array):
    X_array_trans = X_array.transpose()
    output_correlations = np.empty((len(X_array_trans), len(X_array_trans)))
    for i in range(len(X_array_trans)):
        for j in range(i+1,len(X_array_trans)):
            correlation_matrix = np.corrcoef(X_array_trans[i],X_array_trans[j])
            output_correlations[i][j] = correlation_matrix[0][1]
    return output_correlations

def most_correlated_features(X,threshold):
    generic_features = ['close', 'high', 'low', 'open', 'volume']
    columns_to_compute = [col for col in X.columns if col not in generic_features]
    text_features = X[columns_to_compute]
    output_correlations = compute_features_correlation(text_features.values)
    output_correlations_df = pd.DataFrame(output_correlations)
    output_correlations_df.columns = text_features.columns
    bool_df = (output_correlations_df.loc[:, :] < threshold) & (output_correlations_df.loc[:, :] > -threshold)
    columns_to_drop = bool_df.columns[bool_df.isin([False]).any()]
    return columns_to_drop


@njit()
def compute_correlation_to_return(X_array, ret_array):
    X_array_trans = X_array.transpose()
    output_correlations = np.empty((1, len(X_array_trans)))
    for i in range(len(X_array_trans)):
        correlation_matrix = np.corrcoef(X_array_trans[i],ret_array)
        output_correlations[0][i] = correlation_matrix[0][1]
    return output_correlations

def features_correlation_to_return(X,y):
    output_correlations = compute_correlation_to_return(X.values,y.values)
    output_correlations_df = pd.DataFrame(output_correlations)
    output_correlations_df.columns = X.columns
    # return (output_correlations_df>0).astype(int).replace({0: -1})
    correlation_to_return = output_correlations_df.values
    correlation_to_return = correlation_to_return.reshape(correlation_to_return.shape[1])
    return correlation_to_return



def get_anova_columns(X,y):
    diff = {}
    # means = X.mean().to_dict()
    medians = X.median().to_dict()
    # means_df = (X[means.keys()] >= list(means.values())).astype(int)
    # means_df['return'] = augmento_df['return']
    medians_df = (X[medians.keys()] >= list(medians.values())).astype(int)
    medians_df['return'] = y
    data_df = medians_df.copy()
    for column in X.columns:
        diff[column] = abs(pd.DataFrame(data_df.groupby(column)['return'].mean()).diff().values[-1][0])
    anova_features = nlargest(10, diff, key=diff.get)
    return anova_features


def get_binarization_threshold(X):
        return X.mean().to_dict()

def binarize_features(X,features_threshold):
    X[list(features_threshold.keys())] = (X[features_threshold.keys()] >= list(features_threshold.values())).astype(int)
    return X


__author__ = "hugo.inzirillo"

from abc import (
    ABCMeta, abstractmethod,abstractproperty
)

from napoleontoolbox.napoleon_config_tools.object.config import Config


class AbstractParserTemplate(metaclass=ABCMeta):

    @classmethod
    def __subclasshook__(cls, subclass):
        return (hasattr(subclass, 'load') and
                callable(subclass.load) and
                hasattr(subclass, 'existing') and
                callable(subclass.existing) and
                hasattr(subclass, 'read_file') and
                callable(subclass.read_file) or
                NotImplemented)

    @abstractmethod
    def _load(self):
        """
        This method will look up the yaml file attached to your loader
        Returns
        -------

        """
        raise NotImplemented

    @abstractmethod
    def _existing(self):
        """
        This method will check the existence of a config file
        Returns
        -------

        """
        raise NotImplemented

    @abstractmethod
    def _read_file(self):
        """
        This method will read the config file yaml
        Returns
        -------

        """
        raise NotImplemented

    @property
    @abstractmethod
    def config(self) -> Config: raise NotImplemented


from napoleontoolbox.forecasting import preprocessing_features
import numpy as np
import pandas as pd

class BestAccuracySelecter():

    def __init__(self):

    def fit(self, X_train, y_train, X_val, y_val):
        pass

    def predict(self, X_test):
        pass


    def get_features_importance(self, features_names):
        pass



#!/usr/bin/env python3
# coding: utf-8

import sqlite3

from multiprocessing import Pool
from napoleontoolbox.file_saver import dropbox_file_saver
import math
import numpy as np
import json

from napoleontoolbox.signal import signal_utility

class CryptoParalleLauncher():
    def __init__(self,drop_token='', dropbox_backup=True, local_root_directory='../data',user='napoleon',  db_path_suffix = '_run.sqlite', low_bounds=[0.], up_bounds=[1.], ss=[10], ns=[21], leverages =[1.], signal_idiosyncratic_paramaters = [None]):

        self.table_max_size = 2000
        self.args = []
        self.key_args = []
        self.mapping_keys = {}

        self.seed = 0
        for s in ss:
            for n in ns:
                for low_bound in low_bounds:
                    for up_bound in up_bounds:
                        for leverage in leverages:
                            for signal_type, params  in signal_idiosyncratic_paramaters.items():
                                for param in params:
                                    self.args.append((self, self.seed, n, s, low_bound, up_bound, leverage, signal_type,json.dumps(param, sort_keys=True)))
                                    self.key_args.append((self.seed, n, s, low_bound, up_bound, leverage, signal_type,json.dumps(param, sort_keys=True)))
        self.key_args.sort()
        self.counter = 1
        for me_arg in self.key_args:
            saving_key, _ = self.create_saving_key(*me_arg)
            self.mapping_keys[saving_key] = self.counter
            self.counter = self.counter + 1
        self.run_number = self.counter - 1
        print('total number run to execute '+str(self.run_number))
        self.nb_tables = math.ceil(self.run_number/self.table_max_size)
        self.local_root_directory = local_root_directory
        self.user = user
        self.db_path_suffix = db_path_suffix
        self.filename =  user + db_path_suffix
        self.db_path = self.local_root_directory + self.filename
        self.runs = []
        self.totalRow = 0
        self.instantiateTables()
        self.empty_runs_to_investigate = []
        self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=drop_token,dropbox_backup=dropbox_backup)

    def create_saving_key(self, seed, n, s, low_bound, up_bound, leverage, signal_type, idios_string):
        idios = json.loads(idios_string)
        common_params ={
            'seed':seed,
            'n':n,
            's':s,
            'low_bound': low_bound,
            'up_bound': up_bound,
            'leverage':leverage,
            'signal_type' : signal_type,
        }
        common_params.update(idios)
        saving_key = json.dumps(common_params, sort_keys=True)
        saving_key = signal_utility.convert_to_sql_column_format(saving_key)
        return saving_key, idios


    def launchParallelPool(self, toRun, use_num_cpu):
        with Pool(processes=use_num_cpu) as pool:
            results = pool.starmap(toRun, self.args)
            print('parallel computation done')

    def launchSequential(self, toRun):
        results=[]
        for meArg in self.args:
            predicted_weights = toRun(*meArg)
            results.append(predicted_weights)
        return results

    def instantiateTables(self):
        for table_counter in range(self.nb_tables):
            print('instantiating table number '+str(table_counter))
            self.instantiateTable(table_counter)

    def instantiateTable(self, table_number):
        sqliteConnection = None
        try:
            sqliteConnection = sqlite3.connect(self.db_path)
            sqlite_create_table_query = 'CREATE TABLE parallel_run_'+str(table_number)+' (effective_date date PRIMARY KEY);'
            cursor = sqliteConnection.cursor()
            print("Successfully Connected to SQLite")
            cursor.execute(sqlite_create_table_query)
            sqliteConnection.commit()
            print("SQLite table created")
            cursor.close()
        except sqlite3.Error as error:
            print("Error while creating a sqlite table", error)
        finally:
            if (sqliteConnection):
                sqliteConnection.close()
                print("sqlite connection is closed")

    def addRun(self, table_number, run):
        sqliteConnection = None
        try:
            sqliteConnection = sqlite3.connect(self.db_path)
            sqlite_create_table_query = 'alter table parallel_run_'+str(table_number)+' add column ' + run + ' real'
            cursor = sqliteConnection.cursor()
            print("Successfully Connected to SQLite")
            cursor.execute(sqlite_create_table_query)
            sqliteConnection.commit()
            print("SQLite table created")
            cursor.close()
            self.runs.append(run)
        except sqlite3.Error as error:
            print("Error while creating a sqlite table", error)
        finally:
            if (sqliteConnection):
                sqliteConnection.close()
                print("sqlite connection is closed")  # adding the column

    def insertResults(self, table_number, run, values):
        sqliteConnection = None
        try:
            success = True
            sqliteConnection = sqlite3.connect(self.db_path)
            cursor = sqliteConnection.cursor()
            for i, v in values.iteritems():
                sqlite_insert_query = """INSERT INTO 'parallel_run_""" +str(table_number) + """' ('effective_date', '""" + run + """') VALUES ('""" + str(
                    i) + """','""" + str(v) + """');"""
                cursor.execute(sqlite_insert_query)
            sqliteConnection.commit()
            cursor.close()
        except sqlite3.Error as error:
            print("Error while working with SQLite", error)
            success = False
        finally:
            if (sqliteConnection):
                sqliteConnection.close()
                print("The Sqlite connection is closed")
        return success

    def updateResults(self, table_number, run, values):
        sqliteConnection = None
        try:
            sqliteConnection = sqlite3.connect(self.db_path)
            cursor = sqliteConnection.cursor()
            for i, v in values.iteritems():
                sqlite_update_query = """UPDATE 'parallel_run_"""+str(table_number)+"""' set '""" + run + """' = '""" + str(
                    v) + """' where effective_date = '""" + str(i) + """'"""
                cursor.execute(sqlite_update_query)
            sqliteConnection.commit()
            cursor.close()
        except sqlite3.Error as error:
            print("Error while working with SQLite", error)
        finally:
            if (sqliteConnection):
                sqliteConnection.close()
                print("The Sqlite connection is closed")


    def addRunWeights(self,run, weights_df):
        sqliteConnection = None
        try:
            table_name = run + '_weight'
            sqliteConnection = sqlite3.connect(self.db_path)
            weights_df.columns = [col.replace(' ','_') for col in weights_df.columns]
            weights_df.to_sql(name=table_name, con=sqliteConnection)
        except sqlite3.Error as error:
            print("Error while creating a sqlite table", error)
        finally:
            if (sqliteConnection):
                sqliteConnection.close()
                print("sqlite connection is closed")

    def addRunWeightsPrediction(self,run, string_date, weights_df):
        sqliteConnection = None
        try:
            table_name = run +'_'+string_date+ '_weight_prediction'
            sqliteConnection = sqlite3.connect(self.db_path)
            weights_df.columns = [col.replace(' ','_') for col in weights_df.columns]
            weights_df.to_sql(name=table_name, con=sqliteConnection)
        except sqlite3.Error as error:
            print("Error while creating a sqlite table", error)
        finally:
            if (sqliteConnection):
                sqliteConnection.close()
                print("sqlite connection is closed")


    def addRunRecordings(self,run, recordings_df):
        sqliteConnection = None
        try:
            table_name = run + '_recording'
            sqliteConnection = sqlite3.connect(self.db_path)
            recordings_df.to_sql(name=table_name, con=sqliteConnection)
        except sqlite3.Error as error:
            print("Error while creating a sqlite table", error)
        finally:
            if (sqliteConnection):
                sqliteConnection.close()
                print("sqlite connection is closed")


    def addRunActivationsFollowUp(self, run, activations_df):
        sqliteConnection = None
        try:
            table_name = run + '_activation'
            sqliteConnection = sqlite3.connect(self.db_path)
            activations_df.to_sql(name=table_name, con=sqliteConnection)
        except sqlite3.Error as error:
            print("Error while creating a sqlite table", error)
        finally:
            if (sqliteConnection):
                sqliteConnection.close()
                print("sqlite connection is closed")

    def addResults(self, table_number, run, values):
        success = self.insertResults(table_number, run, values)
        if not success :
            self.updateResults(table_number, run, values)

    def getRunAssociatedTable(self, run):
        run_counter = self.mapping_keys[run]
        table_number = run_counter % self.nb_tables
        return table_number

    def saveResults(self, run, values, weights_df):
        table_number = self.getRunAssociatedTable(run)
        self.addRun(table_number,run)
        self.addResults(table_number, run, values)
        #self.totalRow = self.datesCount()
        self.addRunWeights(run, weights_df)


    def checkRunExistence(self,run):
        for table_number in range(self.nb_tables):
            if self.checkLocalRunExistence(table_number,run):
                return True, table_number
        return False, np.nan

    def checkLocalRunExistence(self,table_number, run):
        sqliteConnection = None
        runExistence=None
        try:
            sqliteConnection = sqlite3.connect(self.db_path)
            sqlite_select_run_query = 'SELECT effective_date,'+run + ' FROM parallel_run_'+str(table_number)
            cursor = sqliteConnection.cursor()
            print("Successfully Connected to SQLite")
            cursor.execute(sqlite_select_run_query)
            sqliteConnection.commit()
            print("SQLite table created")
            cursor.close()
            runExistence = True
        except sqlite3.Error as error:
            print("The run is not present", error)
            runExistence = False
        finally:
            if (sqliteConnection):
                sqliteConnection.close()
                print("sqlite connection is closed")  # adding the column
        return runExistence

    def datesCount(self):
        sqliteConnection = None
        try:
            sqliteConnection = sqlite3.connect(self.db_path, timeout=20)
            cursor = sqliteConnection.cursor()
            sqlite_select_query = """SELECT count(*) from parallel_run"""
            cursor.execute(sqlite_select_query)
            totalRows = cursor.fetchone()
            print("Total rows are:  ", totalRows)
            cursor.close()
            return totalRows
        except sqlite3.Error as error:
            print("Failed to read data from sqlite table", error)
        finally:
            if (sqliteConnection):
                sqliteConnection.close()
                print("The Sqlite connection is closed")



    def upload_results_to_dropbox(self):
        print('uploading to dropbox the results '+self.db_path)
        self.dbx.uploadFileToDropbox(filename=self.filename,fullpath = self.db_path)



import pandas as pd

def shift_daily_crypto_compare_signal(hourly_data = None, ssj=None, local_root_directory=None, shift = 1, starting_date=None, running_date=None, frequence='daily'):

    hourly_data = hourly_data.drop_duplicates()

    hourly_data['date'] = pd.to_datetime(hourly_data.index)
    hourly_data['datetime'] = hourly_data.index.date
    hourly_data['hour'] = hourly_data.index.hour

    hourly_data['trigger']  = hourly_data['hour'] == shift
    hourly_data['datetime'] = hourly_data['datetime'].shift(shift)
    hourly_data['trigger'] = hourly_data['trigger'].cumsum()
    hourly_data = hourly_data.dropna()
    # aggregate by datetime and trigger : it should be the same
    hourly_data = hourly_data.groupby(['datetime']).agg({'date':'min', 'close': 'last', 'high': 'max', 'low': 'min', 'open': 'first', 'volumefrom': 'sum', 'volumeto' : 'sum'})
    hourly_data = hourly_data.drop_duplicates()
    hourly_data.index = hourly_data['date']
    hourly_data = hourly_data.drop(columns = ['date'])

    return hourly_data







#!/usr/bin/env python3
# coding: utf-8


""" Object to analyze the market. """

# Built-in packages

# Third party packages
from napoleontoolbox.utility import metrics
import numpy as np
import pandas as pd
import seaborn as sns
from IPython.display import display
from IPython.display import Image
from matplotlib import pyplot as plt


# Local packages
from napoleontoolbox.tools.time_tools import _parse_date, parser_date
from napoleontoolbox.tools.analyze_tools import roll_corr
from napoleontoolbox.signal import signal_utility
__all__ = ['MarketAnalyzer']


class MarketAnalyzer:
    """ Analyze and compare each other asset indices.

    Parameters
    ----------
    data : pd.DataFrame
        Dataframe of market data.
    base : float, optional
        Base to compare asset performances, default is 100.

    Methods
    -------
    set_index
    _set_assets
    _set_metric
    get_perf
    get_corr
    get_kpi

    Attributes
    ----------
    df, portfolio : pd.DataFrame
        Observations on which the metrics are computed, analyzed and compared.
    idx : array-like
        Period over which the metrics are computed, analyzed and compared.
    assets : list of str
        Assets on which the metrics are computed, analyzed and compared.
    metrics : list of str
        List of metrics to compute.

    """

    # Data attribute
    df = pd.DataFrame()
    portfolio = pd.DataFrame()
    idx = []
    # Asset attribute
    assets = []
    n = 0
    # Metric attribute
    # TODO : Possible to simplify
    parser_metrics = {
        'sharpe': {'f': metrics.sharpe, 'p': {'period': 252}},
        'calmar': {'f': metrics.calmar, 'p': {'period': 252}},
        'annual_return': {'f': metrics.annual_return, 'p': {'period': 252}},
        'annual_volatility': {'f': metrics.annual_volatility, 'p': {'period': 252}},
        #'z_score': {'f': metrics.z_score, 'p': {'lags': 63}},
        'mdd': {'f': metrics.mdd, 'p': {}},
    }
    metrics = []
    m = 0
    # Set color palette
    green_red = sns.diverging_palette(17, 170, n=5, l=70, as_cmap=True)
    red_green = sns.diverging_palette(170, 17, n=5, l=70, as_cmap=True)

    def __init__(self, data, base=100., hourlyze = False):
        """ Initialize object. """
        # Set data attribute
        self.base = base
        self.DF = data
        self.T, self.N = data.shape
        self.IDX = data.index
        # Set date attribute
        self._set_index(min(self.IDX), max(self.IDX))
        # Set assets attribute
        self._set_assets(list(data.columns))
        # Set metrics attribute
        # TODO : available_metrics
        self.set_metric('annual_return', 'annual_volatility', 'sharpe',
                        'calmar', 'mdd')#, 'z_score')
        self.hourlyze = hourlyze
    def _set_days_per_year(self):
        # Compute the average of days per year
        self.n_day_per_year = int(
            self.idx.size * 365 / (self.idx[-1] - self.idx[0]).days
        )
        #self.n_day_per_year = 252
        # t = self.df.shape[0]
        # Update parameters of parser_metrics
        for m in self.parser_metrics.keys():
            if 'period' in self.parser_metrics[m]['p'].keys():
                self.parser_metrics[m]['p']['period'] = 365#self.n_day_per_year

            # if 'lags' in self.parser_metrics[m]['p'].keys():
            #    self.parser_metrics[m]['p']['lags'] = min(63, max(1, t - 3))
            #    #m['p']['lags'] = min(self.n_day_per_year // 4, self.idx.size)

    def get_index(self, start, end=None):
        """ Get selected index from `start` to `end` period.

        Parameters
        ----------
        start : str, int or datetime.date
            - Select the first period with int, datetime.date or str with
                format 'YYYY-MM-DD'.
            - Select a period until to `end` with str as {'ytd', 'qtd', 'mtd',
                'y', 'q', 'm', 'w', 'd'} and you can append an int before the
                period, e.g '3y', '8w', etc.
        end : str, int or datetime.date
            Last period to select, must be match with format of dataframe
            index. Default is the last period available.

        Returns
        -------
        pd.Index
            The selected index period.

        """
        if end is None:
            end = self.IDX[-1]

        end = _parse_date(end, self.IDX)

        # if start is a period convert it to a date
        if isinstance(start, str) and '-' not in start:
            start = parser_date(end, period=start)

        start = _parse_date(start, self.IDX)

        if start >= end:

            raise ValueError(
                "{} is greater or equal than {}.".format(start, end)
            )

        return self.IDX[(self.IDX >= start) & (self.IDX <= end)]

    def _set_data(self, idx, assets):
        """ Set dataframe on which metrics are computed and analyzed. """
        if self.n >= self.N:
            assets = slice(None)

        if idx.size >= self.IDX.size:
            idx = slice(None)

        self.df = (self.DF.loc[idx, assets]
                          .dropna(axis=1, how='all')
                          .dropna(axis=0, how='any')
                          .copy())
        self.idx = self.df.index

    def set_perimeter(self, assets, start, end=None):
        self._set_assets(assets)
        self._set_index(start,end)

    def _set_index(self, start, end=None):
        """ Select starting and ending period to analyze.

        Parameters
        ----------
        start : str, int or datetime.date
            - Select the first period with int, datetime.date or str with
                format 'YYYY-MM-DD'.
            - Select a period until to `end` with str as {'ytd', 'qtd', 'mtd',
                'y', 'q', 'm', 'w', 'd'} and you can append an int before the
                period, e.g '3y', '8w', etc.
        end : str, int or datetime.date
            Last period to select, must be match with format of dataframe
            index. Default is the last period available.

        """
        try:
            idx = self.get_index(start, end)

        except ValueError:
            pass

        else:
            self._set_data(idx, self.assets)
            self._set_days_per_year()

    def _set_assets(self, assets):
        """ Set assets to analyze and compare.

        Parameters
        ----------
        assets : list of str
            Name of assets to select in `DF`.

        """
        self.assets = assets
        self.n = len(assets)
        self._set_data(self.idx, self.assets)

    def set_metric(self, *metrics):
        """ Set metrics to compute.

        Parameters
        ----------
        *metrics : {'annual_return', 'annual_volatility', 'calmar', 'mdd', \
                'sharpe', 'z_score'}
            Name of metrics to compute.

        """
        # TODO : verify if metrics are available
        self.metrics = metrics
        self.m = len(metrics)


    def get_kpi(self, drop_duplicate = True, only_one_per_type = False):
        """ Compute some key performance indicators and set them in a table.

        Returns
        -------
        pd.DataFrame
            Table of key performance indicators.

        """
        def filter_daily(dataframe):
            dataframe['hour']=dataframe.index.hour
            dataframe=dataframe[dataframe['hour']==0]
            dataframe = dataframe.drop(columns = ['hour'])
            return dataframe


        comp_df = None
        if self.hourlyze:
            comp_df = filter_daily(self.df.copy())
        else :
            comp_df = self.df.copy()

        results = np.zeros([self.m, self.n])
        # df = self.get_perf()

        i = 0
        for m in self.metrics:
            j = 0
            f, p = (self.parser_metrics[m][k] for k in ['f', 'p'])
            if 'lags' in p.keys():
                # print(p)
                p['lags'] = min(63, max(comp_df.shape[0] - 1, 1))
                # print(p)

            for a in self.assets:
                try:
                    results[i, j] = f((getattr(comp_df, a).values), **p)
                    j += 1
                except Exception as inst:
                    print(inst)
                    continue

            i += 1

        kpis_df = pd.DataFrame(results, columns=self.assets, index=self.metrics)

        if drop_duplicate:
            print('dropping duplicate')
            def suppress_dup_cols(dataf):
                filtered_dataf = dataf.copy()
                all_columns = list(dataf.columns)

                for col_one in all_columns:
                    identical_mapping = set()
                    if col_one in filtered_dataf.columns:
                        for col_two in filtered_dataf.columns:
                            s1 = filtered_dataf[col_one]
                            s2 = filtered_dataf[col_two]
                            if not col_one == col_two:
                                if ((s1 == s2).all()):
                                    identical_mapping.add(col_two)
                        print('dropping ' + str(identical_mapping))
                    filtered_dataf = filtered_dataf.drop(columns=identical_mapping)
                return filtered_dataf

            filtered_kpis_df = suppress_dup_cols(kpis_df.T.copy())

            print('duplicated dropped')
            kpis_df = filtered_kpis_df.T.copy()
        if only_one_per_type :
            print('keeping only the best per filter type')
            kpis_df = kpis_df.T
            kpis_df['label'] = kpis_df.index

            def get_algo_type(row):
                type = ''
                try:
                    type = signal_utility.unjsonize_list_run_results_list([row['label']])[0]['signal_type']
                except Exception as err:
                    print("Parsing signal type error: {0}".format(err))
                return type

            kpis_df['label_type'] = kpis_df.apply(get_algo_type, axis=1)



            def only_the_bests(group):
                sorted_values = group.sort_values(by='calmar')
                return sorted_values.tail(1)

            signal_grouped_kpis = kpis_df.groupby('label_type').apply(only_the_bests)

            signal_grouped_kpis.index = signal_grouped_kpis.label

            kpis_df = signal_grouped_kpis.copy()
            kpis_df = kpis_df.T
            kpis_df = kpis_df[~kpis_df.index.isin(['label', 'label_type'])]

            # return set_table(results, self.assets, self.metrics)
        return kpis_df

    def display_kpi(self, title='KPI table'):
        """ Display KPI. """
        df = self.get_kpi().T
        df.index.name = title

        if 'mdd' in self.metrics:
            df.mdd = - df.mdd

        style = df.style.background_gradient(
            cmap=self.green_red,
            subset=[a for a in self.metrics if a != 'annual_volatility']
        ).background_gradient(
            cmap=self.red_green,
            subset=[a for a in self.metrics if a == 'annual_volatility']
        ).format({
            'annual_return': '{:.2%}',
            'annual_volatility': '{:.2%}',
            'sharpe': '{:.2f}',
            'calmar': '{:.2f}',
            'mdd': '{:.2%}',
            'z_score': '{:.2f}',
        })

        display(style)



    def get_perf(self):
        """ Get asset performances.

        Returns
        -------
        pd.DataFrame
            Performance of assets.

        """
        return self.base * self.df / self.df.iloc[0, :].values

    def plot_perf(self, title='Performances', ax=None, show=True,saving_png_path = None, **kwargs):
        """ Plot asset performances.

        Parameters
        ----------
        title : str, optional
            Title to display on the graph, default is 'Performances'.
        ax : matplotlib.Axes, optional
            Axes object to plot the performances. Default is None it will
            create a new one ax object.
        show : bool, optional
            If true display the figure. May useful to update already existing
            ax object. Default is True.
        **kwargs
            Keyword arguments passed to ``plot`` method.

        """
        df = self.get_perf()

        if ax is None:
            f, ax = plt.subplots(1, 1, figsize=(16, 16))

        if not df.empty:
            df.plot(ax=ax, title=title, **kwargs)


        if show:
            plt.show()

        if saving_png_path is not None:
            plt.savefig(saving_png_path, format='png')

    def get_corr(self, window):
        """ Get rolling cross-correlation.

        Returns
        -------
        pd.DataFrame
            Rolling pairwise correlation of assets.

        """
        if self.n < 2:
            return pd.DataFrame()

        if isinstance(window, str):
            w = int(window[:-1])
        else:
            w = int(window)

        i = self.IDX[max(np.argmax(self.IDX == self.idx[0])-w,0)]

        idx = self.get_index(i, self.idx[-1])
        df = self.DF.loc[idx, self.assets]

        return roll_corr(df, window=window).loc[self.idx, :].dropna()

    def get_stacked_corr(self, window):
        """ Get rolling cross-correlation.
        Returns
        -------
        pd.DataFrame
            Rolling pairwise correlation of assets.
        """
        if self.n < 2:
            return pd.DataFrame()

        if isinstance(window, str):
            w = int(window[:-1])
        else:
            w = int(window)
        i = self.IDX[max(np.argmax(self.IDX == self.idx[0])-w,0)]
        idx = self.get_index(i, self.idx[-1])
        df = self.DF.loc[idx, self.assets]
        mat_roll_cor = df.rolling(window).corr()
        return mat_roll_cor


#!/usr/bin/env python3
# coding: utf-8

""" Module with some function plot backtest. """

# Built-in packages

# External packages
import numpy as np
from matplotlib import pyplot as plt
import seaborn as sns

# Local packages
from napoleontoolbox.backtest.plot_backtest import PlotBackTest

# Set plot style
plt.style.use('seaborn')

__all__ = ['DynaPlotBackTest']


# TODO : FINISH DOCSTRING
class DynaPlotBackTest(PlotBackTest):
    """ Dynamic plot backtest object.

    Attributes
    ----------
    fig : matplotlib.figure.Figure
        Figure to display backtest.
    ax : matplotlib.axes
        Axe(s) to display a part of backtest.

    Methods
    -------
    plot(y, x=None, names=None, col='Blues', lw=1., **kwargs)
        Plot performances.

    See Also
    --------
    PlotBackTest, display_perf, set_text_stats

    """

    plt.ion()
    # def _set_figure(self, fig, ax, size):
    #    """ Set figure, axes and parameters for dynamic plot. """
    #    PlotBackTest._set_figure(self, fig, ax, size)
    #    plt.ion()
    #    return self

    def plot(self, y, x=None, names=None, col='Blues', lw=1., unit='raw',
             **kwargs):
        """ Dynamic plot performances.

        Parameters
        ----------
        y : np.ndarray[np.float64, ndim=2], with shape (`T`, `N`)
            Returns or indexes.
        x : np.ndarray[ndim=2], with shape (`T`, 1), optional
            x-axis, can be series of int or dates or string.
        names : str, optional
            Names y lines for legend.
        col : str, optional
            Color of palette, cf seaborn documentation [2]_.
            Default is 'Blues'.
        lw : float, optional
            Line width of lines.
        kwargs : dict, optional
            Parameters for `ax.legend` method, cf matplotlib
            documentation [3]_.

        Returns
        -------
        pbt : PlotBackTest
            Self object.

        References
        ----------
        .. [2] https://seaborn.pydata.org/api.html
        .. [3] https://matplotlib.org/api/axes_api.html#matplotlib.axes.Axes

        """
        # Set data
        T, N = y.shape
        if x is None:
            x = np.arange(T)

        if names is None:
            names = [r'$Model_{}$'.format(i) for i in range(N)]

        elif isinstance(names, str):
            names = [r'${}_{}$'.format(names, i) for i in range(N)]

        col = sns.color_palette(col, N)

        # Set graphs
        h = self.ax.plot(x, y, LineWidth=lw)

        # Set lines
        for i in range(N):
            h[i].set_color(col[i])
            h[i].set_label(self._set_name(names[i], y[:, i], unit=unit))

        # display
        self.ax.legend(**kwargs)
        # self.fig.canvas.draw()

        return self

    def _set_name(self, name, y, unit='raw', **kwargs):
        if unit.lower() == 'raw':

            return '{}: {:.2f}'.format(name, y[-1])

        elif unit.lower() == 'perf':

            return '{}: {:.0%}'.format(name, y[-1] / y[0] - 1)

        else:

            raise ValueError



__author__ = "hugo.inzirillo"

from dataclasses import dataclass


@dataclass(frozen=False)
class PortfolioDecomposition:
    ts: str
    underlying: str
    weight: float
    quantity: float
    close: float
    usd_amount:float=None
    def __post_init__(self):
        self.usd_amount = self.quantity * self.close

    def __eq__(self, other):
        return self.underlying == other.underlying and self.ts == other.ts if isinstance(other,
                                                                                         self.__class__) else False

    def __ne__(self, other):
        return self.underlying != other.underlying and self.ts != other.ts if isinstance(other,
                                                                                         self.__class__) else True

    def __le__(self, other):
        return self.ts <= other.ts

    def __ge__(self, other):
        return self.ts >= other.ts

    def __lt__(self, other):
        return self.ts < other.ts

    def __gt__(self, other):
        return self.ts > other.ts



__author__ = "hugo.inzirillo"

from napoleontoolbox.napoleon_config_tools.object.modelparamtemplate import ModelParamTemplate


# todo exemple mais cette classe sera à sortir et à déplacer dans un repertoire qui contient tous les modèles de params
# todo faire la config complete

class ModelParamCointegration(ModelParamTemplate):
    def __init__(self):
        super(ModelParamCointegration, self).__init__()
        self.__alpha = None
